#include "wxUI.h"

wxString months[12] ;

wxUI::wxUI(KissCount* kiss, const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame(NULL, -1, title, pos, size), _kiss(kiss), _accountPanel(NULL), _preferencesPanel(NULL), _curPanel(NULL),
    _locale(NULL), _needReload(false)
{
  _hbox = new wxBoxSizer(wxVERTICAL);
  ButtonPanel* buttons = new ButtonPanel(_kiss, this);
  // wxMenu *menuFile = new wxMenu;

  // menuFile->Append( ID_About, wxT("&About...") );
  // menuFile->AppendSeparator();
  // menuFile->Append( ID_Quit, wxT("E&xit") );

  // wxMenuBar *menuBar = new wxMenuBar;
  // menuBar->Append( menuFile, wxT("&File") );

  // SetMenuBar( menuBar );

  // CreateStatusBar();
  // SetStatusText( wxT("Welcome to wxWidgets!") );

  SetSizer(_hbox);

  _hbox->Add(buttons);
}

wxUI::~wxUI()
{
  if (_accountPanel) delete _accountPanel;  
  if (_preferencesPanel) delete _preferencesPanel;
  if (_locale) delete _locale;
}

bool wxUI::SetLanguage(long language)
{
  bool res = true;

  if (_locale) delete _locale;
  _locale = NULL;

  // load language if possible, fall back to english otherwise
  if(wxLocale::IsAvailable(language))
    {
      _locale = new wxLocale( language, wxLOCALE_CONV_ENCODING );
 
#ifdef __WXGTK__
      _locale->AddCatalogLookupPathPrefix(wxT("./ressources/po"));
#endif
 
      _locale->AddCatalog(wxT("french"));
      _locale->AddCatalog(wxT("kisscount"));

      _language = (wxLanguage) language;
    }

  if (_locale == NULL || !_locale->IsOk())
    {
      if (_locale) delete _locale;
      _locale = new wxLocale();

#ifdef __WXGTK__
      _locale->AddCatalogLookupPathPrefix(wxT("./ressources/po"));
#endif

      _locale->AddCatalog(wxT("kisscount"));
      
      _language = wxLANGUAGE_ENGLISH;
      res = false;
    }

  months[0] = _("january");
  months[1] = _("february");
  months[2] = _("march");
  months[3] = _("april");
  months[4] = _("may");
  months[5] = _("june");
  months[6] = _("july");
  months[7] = _("august");
  months[8] = _("september");
  months[9] = _("october");
  months[10] = _("november");
  months[11] = _("december") ;

  return res;
}

void wxUI::ShowAccount()
{
  ShowPanel(_accountPanel);
}

void wxUI::ShowPreferences()
{
  ShowPanel(_preferencesPanel);
}

void wxUI::ChangeUser()
{
  UsersDialog u(_kiss, this);
  u.ShowModal();
}

void wxUI::LoadUser()
{
  User* user = _kiss->GetUser();

  if (_curPanel)
    {
      _hbox->Detach(_curPanel);
      _curPanel = NULL;
    }

  if (_accountPanel) 
    delete _accountPanel;

  if (_preferencesPanel)
    delete _preferencesPanel;

  if (user->_preferences[wxT("language")] != wxT(""))
    SetLanguage(user->GetLanguage());

  _preferencesPanel = new PreferencesPanel(_kiss, this);
  _accountPanel = new AccountPanel(_kiss, this);

  ShowPanel(_accountPanel);
}

void wxUI::ShowPanel(wxPanel* panel)
{
  int month, year, account=0, preferences=0;
  wxShowEvent event;

  if (!panel) return;

  if (_curPanel)
    {
      _hbox->Detach(_curPanel);
      _curPanel->Hide();
    }

  if (_needReload)
    {
      if (panel == _accountPanel)
      	{
	  account = 1;
      	  month = _accountPanel->_curMonth;
      	  year = _accountPanel->_curYear;
      	}
            
      if (panel == _preferencesPanel)
      	{
	  preferences = 1;
      	}

	delete _accountPanel;
	delete _preferencesPanel;

	_accountPanel = new AccountPanel(_kiss, this);
	if (year != -1)
	  _accountPanel->ShowMonth(month, year);
	_preferencesPanel = new PreferencesPanel(_kiss, this);

	if (account) 
	  {
	    _accountPanel->OnShow(event);
	    panel = _accountPanel;
	  }
	if (preferences) 
	  {
	    _preferencesPanel->OnShow(event);
	    panel = _preferencesPanel;
	  }
	
      _needReload = false;
    }

  _curPanel = panel;
  _hbox->Add(panel);
  _curPanel->Show();
  Layout();
}

void wxUI::GenerateMonth(int month, int year)
{
  _accountPanel->GenerateMonth(month, year);
}

void wxUI::KillMe()
{
  if (_curPanel)
    {
      _hbox->Detach(_curPanel);
      _curPanel = NULL;
    }

  if (_accountPanel) 
    delete _accountPanel;

  if (_preferencesPanel)
    delete _preferencesPanel;

  _accountPanel = NULL;
  _preferencesPanel = NULL;
}

void wxUI::NeedReload()
{
  _needReload = true;
}
